@@ -2,9 +2,10 @@
     This file is part of the KDE libraries
 
     Copyright (C) 1998 Lars Knoll (knoll@mpi-hd.mpg.de)
-    Copyright (C) 2001-2003 Dirk Mueller (mueller@kde.org)
-    Copyright (C) 2002 Waldo Bastian (bastian@kde.org)
-    Copyright (C) 2003 Apple Computer, Inc.
+              (C) 2001-2003 Dirk Mueller (mueller@kde.org)
+              (C) 2002 Waldo Bastian (bastian@kde.org)
+              (C) 2003 Apple Computer, Inc.
+              (C) 2006-2010 Germain Garand (germain@ebooksfrance.org)
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -44,6 +45,7 @@
 
 #include "loader.h"
 #include "seed.h"
+#include "woff.h"
 #include <imload/image.h>
 #include <imload/imagepainter.h>
 #include <kfilterdev.h>
@@ -52,7 +54,6 @@
 
 // default cache size
 #define DEFCACHESIZE 2096*1024
-#define MAX_JOB_COUNT 32
 
 //#include <qasyncio.h>
 //#include <qasyncimageio.h>
@@ -241,8 +242,12 @@ CachedCSSStyleSheet::CachedCSSStyleSheet(DocLoader* dl, const DOMString &url, KI
     m_hadError = false;
     m_wasBlocked = false;
     m_err = 0;
-    // load the file
-    Cache::loader()->load(dl, this, false, true /*highPriority*/);
+    // load the file.
+    // Style sheets block rendering, they are therefore the higher priority item.
+    // Do |not| touch the priority value unless you conducted thorough tests and
+    // can back your choice with meaningful data, testing page load time and
+    // time to first paint.
+    Cache::loader()->load(dl, this, false, -8);
     m_loading = true;
 }
 
@@ -347,8 +352,10 @@ CachedScript::CachedScript(DocLoader* dl, const DOMString &url, KIO::CacheContro
     // But some websites think their scripts are <some wrong mimetype here>
     // and refuse to serve them if we only accept application/x-javascript.
     setAccept( QLatin1String("*/*") );
-    // load the file
-    Cache::loader()->load(dl, this, false);
+    // load the file.
+    // Scripts block document parsing. They are therefore second in our list of most
+    // desired resources.
+    Cache::loader()->load(dl, this, false/*incremental*/, -6);
     m_loading = true;
     m_hadError = false;
 }
@@ -417,12 +424,7 @@ CachedImage::CachedImage(DocLoader* dl, const DOMString &url, KIO::CacheControl
     bg = 0;
     scaled = 0;
     bgColor = qRgba( 0, 0, 0, 0 );
-    typeChecked = false;
-    isFullyTransparent = false;
-    monochrome = false;
-    formatType = 0;
     m_status = Unknown;
-    imgSource = 0;
     setAccept( acceptHeader );
     i->setShowAnimations(dl->showAnimations());
     m_loading = true;
@@ -653,6 +655,7 @@ void CachedImage::imageHasGeometry(khtmlImLoad::Image* /*img*/, int width, int h
 #ifdef LOADER_DEBUG
     kDebug(6060) << this << " got geometry "<< width << "x" << height;
 #endif
+
     do_notify(QRect(0, 0, width, height));
 }
 
@@ -662,6 +665,7 @@ void CachedImage::imageChange     (khtmlImLoad::Image* /*img*/, QRect region)
     kDebug(6060) << "Image " << this << " change " <<
         region.x() << "," << region.y() << ":" << region.width() << "x" << region.height() << endl;
 #endif
+
     //### this is overly conservative -- I guess we need to also specify reason,
     //e.g. repaint vs. changed !!!
     delete bg;
@@ -712,125 +716,12 @@ void CachedImage::do_notify(const QRect& r)
     }
 }
 
-
-// void CachedImage::movieUpdated( const QRect& r )
-// {
-// #ifdef LOADER_DEBUG
-//     qDebug("movie updated %d/%d/%d/%d, pixmap size %d/%d", r.x(), r.y(), r.right(), r.bottom(),
-//            m->framePixmap().size().width(), m->framePixmap().size().height());
-// #endif
-//
-//     do_notify(m->framePixmap(), r);
-// }
-#if 0
-
-void CachedImage::movieStatus(int status)
-{
-#warning QMovie emits different signals now and requires different ways to call
-#ifdef LOADER_DEBUG
-    qDebug("movieStatus(%d)", status);
-#endif
-
-    // ### the html image objects are supposed to send the load event after every frame (according to
-    // netscape). We have a problem though where an image is present, and js code creates a new Image object,
-    // which uses the same CachedImage, the one in the document is not supposed to be notified
-
-    // just another Qt 2.2.0 bug. we cannot call
-    // QMovie::frameImage if we're after QMovie::EndOfMovie
-    if(status == QMovie::EndOfFrame)
-    {
-        const QImage& im = m->frameImage();
-        monochrome = ( ( im.depth() <= 8 ) && ( im.numColors() - int( im.hasAlphaBuffer() ) <= 2 ) );
-        for (int i = 0; monochrome && i < im.numColors(); ++i)
-            if (im.colorTable()[i] != qRgb(0xff, 0xff, 0xff) &&
-                im.colorTable()[i] != qRgb(0x00, 0x00, 0x00))
-                monochrome = false;
-        if( (im.width() < 5 || im.height() < 5) && im.hasAlphaBuffer()) // only evaluate for small images
-        {
-            QImage am = im.createAlphaMask();
-            if(am.depth() == 1)
-            {
-                bool solid = false;
-                for(int y = 0; y < am.height(); y++)
-                    for(int x = 0; x < am.width(); x++)
-                        if(am.pixelIndex(x, y)) {
-                            solid = true;
-                            break;
-                        }
-                isFullyTransparent = (!solid);
-            }
-        }
-
-        // we have to delete our tiled bg variant here
-        // because the frame has changed (in order to keep it in sync)
-        delete bg;
-        bg = 0;
-    }
-
-    if((status == QMovie::EndOfMovie && (!m || m->frameNumber() <= 1)) ||
-       ((status == QMovie::EndOfLoop) && (m_showAnimations == KHTMLSettings::KAnimationLoopOnce)) ||
-       ((status == QMovie::EndOfFrame) && (m_showAnimations == KHTMLSettings::KAnimationDisabled))
-      )
-    {
-        if(imgSource)
-        {
-            setShowAnimations( KHTMLSettings::KAnimationDisabled );
-
-            // monochrome alphamasked images are usually about 10000 times
-            // faster to draw, so this is worth the hack
-            if (p && monochrome && p->depth() > 1)
-            {
-                QPixmap* pix = new QPixmap;
-                pix->convertFromImage( p->convertToImage().convertDepth( 1 ), Qt::MonoOnly|Qt::AvoidDither );
-                pix->setMask( p->mask() );
-                delete p;
-                p = pix;
-                monochrome = false;
-            }
-        }
-        for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
-            it.next().value()->notifyFinished( this );
-	m_status = Cached; //all done
-    }
-
-    if((status == QMovie::EndOfFrame) || (status == QMovie::EndOfMovie))
-    {
-#ifdef LOADER_DEBUG
-        QRect r(valid_rect());
-        qDebug("movie Status frame update %d/%d/%d/%d, pixmap size %d/%d", r.x(), r.y(), r.right(), r.bottom(),
-               pixmap().size().width(), pixmap().size().height());
-#endif
-        do_notify(pixmap(), valid_rect());
-    }
-}
-#endif
-
-// void CachedImage::movieResize(const QSize& /*s*/)
-// {
-//     do_notify(m->framePixmap(), QRect());
-// }
-
 void CachedImage::setShowAnimations( KHTMLSettings::KAnimationAdvice showAnimations )
 {
     if (i)
         i->setShowAnimations(showAnimations);
 }
 
-// void CachedImage::deleteMovie()
-// {
-//     delete m; m = 0;
-// }
-
-void CachedImage::pauseAnimations()
-{
-//    if ( m ) m->setPaused( true );
-}
-
-void CachedImage::resumeAnimations()
-{
-//    if ( m ) m->setPaused( false );
-}
-
 void CachedImage::clear()
 {
     delete i;   i = new khtmlImLoad::Image(this);
@@ -839,12 +730,7 @@ void CachedImage::clear()
     delete bg;  bg = 0;
     bgSize = QSize(-1,-1);
 
-    formatType = 0;
-    typeChecked = false;
     setSize(0);
-
-    // No need to delete imageSource - QMovie does it for us
-    imgSource = 0;
 }
 
 void CachedImage::data ( QBuffer &_buffer, bool eof )
@@ -858,87 +744,11 @@ void CachedImage::data ( QBuffer &_buffer, bool eof )
 
     if (eof)
         i->processEOF();
-
-    //if ( !typeChecked )
-    //{
-        // don't attempt incremental loading if we have all the data already
-      //  assert(!eof);
-
-#ifdef __GNUC__
-#warning QImage* requires heavy porting
-#endif
-#if 0
-        formatType = QImageDecoder::formatName( (const uchar*)_buffer.buffer().data(), _buffer.size());
-        if ( formatType && strcmp( formatType, "PNG" ) == 0 )
-            formatType = 0; // Some png files contain multiple images, we want to show only the first one
-
-        typeChecked = true;
-
-        if ( formatType )  // movie format exists
-        {
-            // imgSource = new ImageSource( _buffer.buffer());
-            // m = new QMovie( imgSource, 8192 );
-	    m = new QMovie( _buffer);
-            m->connectUpdate( this, SLOT( movieUpdated( const QRect &) ));
-            m->connectStatus( this, SLOT( movieStatus(int)));
-            m->connectResize( this, SLOT( movieResize( const QSize& ) ) );
-        }
-    }
-
-    if ( imgSource )
-    {
-        imgSource->setEOF(eof);
-        imgSource->maybeReady();
-    }
-
-    if(eof)
-    {
-        // QMovie currently doesn't support all kinds of image formats
-        // so we need to use a QPixmap here when we finished loading the complete
-        // picture and display it then all at once.
-        if(typeChecked && !formatType)
-        {
-#ifdef CACHE_DEBUG
-            kDebug(6060) << "CachedImage::data(): reloading as pixmap:";
-#endif
-            p = new QPixmap;
-            {
-            	QBuffer buffer(_buffer.buffer());
-            	buffer.open(IO_ReadOnly);
-                QImageIO io( &buffer, 0 );
-                io.setGamma(2.2); // hardcoded "reasonable value"
-                bool result = io.read();
-                if (result) p->convertFromImage(io.image(), 0);
-            }
-
-            // set size of image.
-#ifdef CACHE_DEBUG
-            kDebug(6060) << "CachedImage::data(): image is null: " << p->isNull();
-#endif
-            if(p->isNull())
-            {
-                m_hadError = true;
-                do_notify(pixmap(), QRect(0, 0, 16, 16)); // load "broken image" icon
-            }
-            else
-                do_notify(*p, p->rect());
-
-            for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
-                it.next().value()->notifyFinished( this );
-            m_status = Cached; //all done
-        }
-#endif
-    //}
 }
 
 void CachedImage::finish()
 {
-    //Status oldStatus = m_status;
     CachedObject::finish();
-/*    if ( oldStatus != m_status ) {
-	const QPixmap &pm = pixmap();
-	do_notify( pm, pm.rect() );
-    }*/
     m_loading = false;
     QSize s = pixmap_size();
     setSize( s.width() * s.height() * 2);
@@ -948,7 +758,6 @@ void CachedImage::finish()
 void CachedImage::error( int /*err*/, const char* /*text*/ )
 {
     clear();
-    typeChecked = true;
     m_hadError = true;
     m_loading = false;
     do_notify(QRect(0, 0, 16, 16));
@@ -962,7 +771,7 @@ CachedSound::CachedSound(DocLoader* dl, const DOMString &url, KIO::CacheControl
     : CachedObject(url, Sound, _cachePolicy, 0)
 {
     setAccept( QLatin1String("*/*") ); // should be whatever phonon would accept...
-    Cache::loader()->load(dl, this, false);
+    Cache::loader()->load(dl, this, false/*incremental*/, 2);
     m_loading = true;
 }
 
@@ -1004,7 +813,10 @@ CachedFont::CachedFont(DocLoader* dl, const DOMString &url, KIO::CacheControl _c
     : CachedObject(url, Font, _cachePolicy, 0)
 {
     setAccept( QLatin1String("*/*") );
-    Cache::loader()->load(dl, this, true /*highPriority*/);
+    // Fonts are desired early because their absence will lead to a page being rendered
+    // with a default replacement, then the text being re-rendered with the new font when it arrives.
+    // This can be fairly disturbing for the reader - more than missing images for instance.
+    Cache::loader()->load(dl, this, false /*incremental*/, -4);
     m_loading = true;
 }
 
@@ -1028,6 +840,29 @@ void CachedFont::data( QBuffer &buffer, bool eof )
         delete dev;
     }
 
+    // handle decoding of WOFF fonts
+    int woffStatus = eWOFF_ok;
+    if (int need = WOFF::getDecodedSize( m_font.constData(), m_font.size(), &woffStatus)) {
+        kDebug(6040) << "***************************** Got WOFF FoNT";
+        m_hadError = true;
+        do {
+            if (WOFF_FAILURE(woffStatus))
+                break;
+            QByteArray wbuffer;
+            wbuffer.resize( need );
+            int len;
+            woffStatus = eWOFF_ok;
+            WOFF::decodeToBuffer(m_font.constData(), m_font.size(), wbuffer.data(), wbuffer.size(), &len, &woffStatus);
+            if (WOFF_FAILURE(woffStatus))
+                break;
+            wbuffer.resize(len);
+            m_font = wbuffer;
+            m_hadError = false;
+        } while (false);
+    } else if (m_font.isEmpty()) {
+        m_hadError = true;
+    }
+    else kDebug(6040) << "******** #################### ********************* NON WOFF font";
     setSize(m_font.size());
 
     m_loading = false;
@@ -1045,16 +880,18 @@ void CachedFont::checkNotify()
 void CachedFont::error( int /*err*/, const char* /*text*/ )
 {
     m_loading = false;
+    m_hadError = true;
     checkNotify();
 }
 
 // ------------------------------------------------------------------------------------------
 
-Request::Request(DocLoader* dl, CachedObject *_object, bool _incremental)
+Request::Request(DocLoader* dl, CachedObject *_object, bool _incremental, int _priority)
 {
     object = _object;
     object->setRequest(this);
     incremental = _incremental;
+    priority = _priority;
     m_docLoader = dl;
 }
 
@@ -1208,21 +1045,46 @@ void DocLoader::printPreloadStats()
 #endif
 }
 
-#define DOCLOADER_SECCHECK(doRedirectCheck) \
-    KUrl fullURL (m_doc->completeURL( url.string() )); \
-    if ( !fullURL.isValid() || \
-         ( m_part && m_part->onlyLocalReferences() && fullURL.protocol() != "file" && fullURL.protocol() != "data") || \
-         ( doRedirectCheck && (m_doc && !KAuthorized::authorizeUrlAction("redirect", m_doc->URL(), fullURL)))) \
+static inline bool securityCheckUrl(const KUrl& fullURL, KHTMLPart* part, DOM::DocumentImpl* doc,
+                                    bool doRedirectCheck, bool isImg)
+{
+    if (!fullURL.isValid())
+        return false;
+    if (part && part->onlyLocalReferences() && fullURL.protocol() != "file" && fullURL.protocol() != "data")
+        return false;
+    if (doRedirectCheck && doc) {
+        if (isImg && part && part->forcePermitLocalImages() && fullURL.protocol() == "file")
+            return true;
+        else
+            return KAuthorized::authorizeUrlAction("redirect", doc->URL(), fullURL);
+    }
+
+    return true;
+}
+
+#define DOCLOADER_SECCHECK_IMP(doRedirectCheck,isImg) \
+    KUrl fullURL(m_doc->completeURL(url.string())); \
+    if (!securityCheckUrl(fullURL, m_part, m_doc, doRedirectCheck, isImg)) \
          return 0L;
 
+#define DOCLOADER_SECCHECK(doRedirectCheck)     DOCLOADER_SECCHECK_IMP(doRedirectCheck, false)
+#define DOCLOADER_SECCHECK_IMG(doRedirectCheck) DOCLOADER_SECCHECK_IMP(doRedirectCheck, true)
+
+bool DocLoader::willLoadMediaElement( const DOM::DOMString &url)
+{
+    DOCLOADER_SECCHECK(true);
+
+    return true;
+}
+
 CachedImage *DocLoader::requestImage( const DOM::DOMString &url)
 {
-    DOCLOADER_SECCHECK(true);
+    DOCLOADER_SECCHECK_IMG(true);
 
     CachedImage* i = Cache::requestObject<CachedImage, CachedObject::Image>( this, fullURL, 0);
 
     if (i && i->status() == CachedObject::Unknown && autoloadImages())
-        Cache::loader()->load(this, i, true);
+        Cache::loader()->load(this, i, true /*incremental*/);
 
     return i;
 }
@@ -1288,7 +1150,7 @@ void DocLoader::setAutoloadImages( bool enable )
             if ( status != CachedObject::Unknown )
                 continue;
 
-            Cache::loader()->load(this, img, true);
+            Cache::loader()->load(this, img, true /*incremental*/);
         }
     }
 }
@@ -1310,117 +1172,56 @@ void DocLoader::setShowAnimations( KHTMLSettings::KAnimationAdvice showAnimation
     }
 }
 
-void DocLoader::pauseAnimations()
-{
-    for ( QSetIterator<CachedObject*> it( m_docObjects ); it.hasNext(); )
-    {
-        CachedObject* cur = it.next();
-        if ( cur->type() == CachedObject::Image )
-        {
-            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( cur ) );
-
-            img->pauseAnimations();
-        }
-    }
-}
-
-void DocLoader::resumeAnimations()
-{
-    for ( QSetIterator<CachedObject*> it( m_docObjects ); it.hasNext(); )
-    {
-        CachedObject* cur = it.next();
-        if ( cur->type() == CachedObject::Image )
-        {
-            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( cur ) );
-
-            img->resumeAnimations();
-        }
-    }
-}
-
 // ------------------------------------------------------------------------------------------
 
 Loader::Loader() : QObject()
 {
-    connect(&m_timer, SIGNAL(timeout()), this, SLOT( servePendingRequests() ) );
-    m_highPriorityRequestPending = 0;
 }
 
 Loader::~Loader()
 {
-    delete m_highPriorityRequestPending;
-    qDeleteAll(m_requestsPending);
     qDeleteAll(m_requestsLoading);
 }
 
-void Loader::load(DocLoader* dl, CachedObject *object, bool incremental, bool highPriority)
+void Loader::load(DocLoader* dl, CachedObject *object, bool incremental, int priority)
 {
-    Request *req = new Request(dl, object, incremental);
-    if (highPriority && !m_highPriorityRequestPending) {
-        m_highPriorityRequestPending = req;
-    } else {
-        if (highPriority) {
-            m_requestsPending.prepend(req);
-        } else {
-            m_requestsPending.append(req);
-        }
-        highPriority = false;
-    }
-
+    Request *req = new Request(dl, object, incremental, priority);
+    scheduleRequest(req);
     emit requestStarted( req->m_docLoader, req->object );
-
-    if (highPriority) {
-        servePendingRequests();
-    } else {
-        m_timer.setSingleShot(true);
-        m_timer.start(0);
-    }
 }
 
-void Loader::servePendingRequests()
+void Loader::scheduleRequest(Request* req)
 {
-    while ( (m_highPriorityRequestPending != 0 || m_requestsPending.count() != 0) && (m_requestsLoading.count() < MAX_JOB_COUNT) )
-    {
-        // get the first pending request
-        Request *req = m_highPriorityRequestPending ? m_highPriorityRequestPending : m_requestsPending.takeFirst();
-
 #ifdef LOADER_DEBUG
   kDebug( 6060 ) << "starting Loader url=" << req->object->url().string();
 #endif
 
-        KUrl u(req->object->url().string());
-        KIO::TransferJob* job = KIO::get( u, KIO::NoReload, KIO::HideProgressInfo /*no GUI*/);
+    KUrl u(req->object->url().string());
+    KIO::TransferJob* job = KIO::get( u, KIO::NoReload, KIO::HideProgressInfo /*no GUI*/);
 
-        job->addMetaData("cache", KIO::getCacheControlString(req->object->cachePolicy()));
-        if (!req->object->accept().isEmpty())
-            job->addMetaData("accept", req->object->accept());
-        if ( req->m_docLoader )
+    job->addMetaData("cache", KIO::getCacheControlString(req->object->cachePolicy()));
+    if (!req->object->accept().isEmpty())
+        job->addMetaData("accept", req->object->accept());
+    if ( req->m_docLoader )
+    {
+        job->addMetaData( "referrer",  req->m_docLoader->doc()->URL().url() );
+         KHTMLPart *part = req->m_docLoader->part();
+        if (part )
         {
-            job->addMetaData( "referrer",  req->m_docLoader->doc()->URL().url() );
-
-            KHTMLPart *part = req->m_docLoader->part();
-            if (part )
-            {
-                job->addMetaData( "cross-domain", part->toplevelURL().url() );
-                if (part->widget())
-                    job->ui()->setWindow (part->widget()->topLevelWidget());
-            }
-        }
-
-        connect( job, SIGNAL( result( KJob * ) ), this, SLOT( slotFinished( KJob * ) ) );
-        connect( job, SIGNAL( mimetype( KIO::Job *, const QString& ) ), this, SLOT( slotMimetype( KIO::Job *, const QString& ) ) );
-        connect( job, SIGNAL( data( KIO::Job*, const QByteArray &)),
-                 SLOT( slotData( KIO::Job*, const QByteArray &)));
-
-        if ( req->object->schedule() )
-            KIO::Scheduler::scheduleJob( job );
-
-        m_requestsLoading.insertMulti(job, req);
-        if (m_highPriorityRequestPending) {
-            m_highPriorityRequestPending = 0;
-            break;
+            job->addMetaData( "cross-domain", part->toplevelURL().url() );
+            if (part->widget())
+                job->ui()->setWindow (part->widget()->topLevelWidget());
         }
     }
+
+    connect( job, SIGNAL(result(KJob*)), this, SLOT(slotFinished(KJob*)) );
+    connect( job, SIGNAL(mimetype(KIO::Job*,QString)), this, SLOT(slotMimetype(KIO::Job*,QString)) );
+    connect( job, SIGNAL(data(KIO::Job*,QByteArray)),
+             SLOT(slotData(KIO::Job*,QByteArray)));
+
+    KIO::Scheduler::setJobPriority( job, req->priority );
+
+    m_requestsLoading.insertMulti(job, req);
 }
 
 void Loader::slotMimetype( KIO::Job *j, const QString& s )
@@ -1498,11 +1299,6 @@ void Loader::slotFinished( KJob* job )
 #endif
 
   delete r;
-
-  if ( (m_highPriorityRequestPending != 0 || m_requestsPending.count() != 0) && (m_requestsLoading.count() < MAX_JOB_COUNT / 2) ) {
-      m_timer.setSingleShot(true);
-      m_timer.start(0);
-  }
 }
 
 void Loader::slotData( KIO::Job*job, const QByteArray &data )
@@ -1525,13 +1321,6 @@ void Loader::slotData( KIO::Job*job, const QByteArray &data )
 int Loader::numRequests( DocLoader* dl ) const
 {
     int res = 0;
-    if (m_highPriorityRequestPending && m_highPriorityRequestPending->m_docLoader == dl)
-        res++;
-
-    foreach( Request* req, m_requestsPending )
-        if ( req->m_docLoader == dl )
-            res++;
-
     foreach( Request* req, m_requestsLoading)
         if ( req->m_docLoader == dl )
             res++;
@@ -1541,27 +1330,6 @@ int Loader::numRequests( DocLoader* dl ) const
 
 void Loader::cancelRequests( DocLoader* dl )
 {
-    if (m_highPriorityRequestPending && m_highPriorityRequestPending->m_docLoader == dl) {
-        CDEBUG << "canceling high priority pending request for " << m_highPriorityRequestPending->object->url().string() << endl;
-        Cache::removeCacheEntry( m_highPriorityRequestPending->object );
-        delete m_highPriorityRequestPending;
-        m_highPriorityRequestPending = 0;
-    }
-
-    QMutableLinkedListIterator<Request*> pIt( m_requestsPending );
-    while ( pIt.hasNext() ) {
-        Request* cur = pIt.next();
-        if ( cur->m_docLoader == dl )
-        {
-            CDEBUG << "canceling pending request for " << cur->object->url().string() << endl;
-            Cache::removeCacheEntry( cur->object );
-            pIt.remove();
-            delete cur;
-        }
-    }
-
-    //kDebug( 6060 ) << "got " << m_requestsLoading.count() << "loading requests";
-
     QMutableHashIterator<KIO::Job*,Request*> lIt( m_requestsLoading );
     while ( lIt.hasNext() )
     {
