@@ -21,6 +21,7 @@
  */
 
 #include "dialog.h"
+#include "private/dialog_p.h"
 
 #include <QPainter>
 #include <QSvgRenderer>
@@ -52,6 +53,7 @@
 #include "plasma/private/extender_p.h"
 #include "plasma/framesvg.h"
 #include "plasma/theme.h"
+#include "plasma/widgets/scrollwidget.h"
 #include "plasma/windoweffects.h"
 
 #ifdef Q_WS_X11
@@ -61,56 +63,6 @@
 namespace Plasma
 {
 
-class DialogPrivate
-{
-public:
-    DialogPrivate(Dialog *dialog)
-            : q(dialog),
-              background(0),
-              view(0),
-              resizeCorners(Dialog::NoCorner),
-              resizeStartCorner(Dialog::NoCorner),
-              moveTimer(0),
-              aspectRatioMode(Plasma::IgnoreAspectRatio),
-              resizeChecksWithBorderCheck(false)
-    {
-    }
-
-    ~DialogPrivate()
-    {
-    }
-
-    void scheduleBorderCheck(bool triggeredByResize = false);
-    void themeChanged();
-    void updateMask();
-    void checkBorders();
-    void checkBorders(bool updateMaskIfNeeded);
-    void updateResizeCorners();
-    int calculateWidthForHeightAndRatio(int height, qreal ratio);
-    Plasma::Applet *applet();
-    void delayedAdjustSize();
-
-    Plasma::Dialog *q;
-
-    /**
-     * Holds the background SVG, to be re-rendered when the cache is invalidated,
-     * for example by resizing the dialogue.
-     */
-    Plasma::FrameSvg *background;
-    QGraphicsView *view;
-    QWeakPointer<QGraphicsWidget> graphicsWidgetPtr;
-    Dialog::ResizeCorners resizeCorners;
-    QMap<Dialog::ResizeCorner, QRect> resizeAreas;
-    int resizeStartCorner;
-    QTimer *moveTimer;
-    QTimer *adjustViewTimer;
-    QTimer *adjustSizeTimer;
-    QSize oldGraphicsWidgetMinimumSize;
-    QSize oldGraphicsWidgetMaximumSize;
-    Plasma::AspectRatioMode aspectRatioMode;
-    bool resizeChecksWithBorderCheck;
-};
-
 void DialogPrivate::scheduleBorderCheck(bool triggeredByResize)
 {
     //kDebug();
@@ -136,7 +88,7 @@ void DialogPrivate::scheduleBorderCheck(bool triggeredByResize)
         QObject::connect(moveTimer, SIGNAL(timeout()), q, SLOT(checkBorders()));
     }
 
-    moveTimer->start(triggeredByResize ? 0 : 200);
+    moveTimer->start(0);
 }
 
 void DialogPrivate::themeChanged()
@@ -148,6 +100,7 @@ void DialogPrivate::themeChanged()
     // when we aren't compositing
     q->setAttribute(Qt::WA_NoSystemBackground, !translucency);
     updateMask();
+    q->update();
 }
 
 void DialogPrivate::updateMask()
@@ -167,28 +120,6 @@ void DialogPrivate::checkBorders()
     checkBorders(true);
 }
 
-Plasma::Applet *DialogPrivate::applet()
-{
-    Extender *extender = qobject_cast<Extender*>(graphicsWidgetPtr.data());
-    Plasma::Applet *applet = 0;
-    if (extender) {
-        if (!extender->d->applet) {
-            return 0;
-        }
-        applet = extender->d->applet.data();
-    } else if (graphicsWidgetPtr) {
-        QObject *pw = graphicsWidgetPtr.data();
-
-        while ((pw = pw->parent())) {
-            applet = dynamic_cast<Plasma::Applet *>(pw);
-            if (applet) {
-                break;
-            }
-        }
-    }
-    return applet;
-}
-
 void DialogPrivate::delayedAdjustSize()
 {
     q->syncToGraphicsWidget();
@@ -205,12 +136,33 @@ void DialogPrivate::checkBorders(bool updateMaskIfNeeded)
     FrameSvg::EnabledBorders borders = FrameSvg::AllBorders;
 
     Extender *extender = qobject_cast<Extender*>(graphicsWidget);
-    Plasma::Applet *applet = this->applet();
+    Plasma::Applet *applet = appletPtr.data();
 
     //used to remove borders at the edge of the desktop
+    QRect avail;
+    QRect screenGeom;
     QDesktopWidget *desktop = QApplication::desktop();
-    QRect avail = desktop->availableGeometry(desktop->screenNumber(q));
-    QRect screenGeom = desktop->screenGeometry(desktop->screenNumber(q));
+    Plasma::Corona *c = 0;
+    if (applet) {
+        c = qobject_cast<Plasma::Corona *>(applet->scene());
+    } else if (graphicsWidget) {
+        c = qobject_cast<Plasma::Corona *>(graphicsWidget->scene());
+    }
+    if (c) {
+        QRegion r = c->availableScreenRegion(desktop->screenNumber(q));
+        QRect maxRect;
+        foreach (QRect rect, r.rects()) {
+            if (rect.width() > maxRect.width() && rect.height() > maxRect.height()) {
+                maxRect = rect;
+            }
+        }
+        avail = maxRect;
+        screenGeom = c->screenGeometry(desktop->screenNumber(q));
+    } else {
+        avail = desktop->availableGeometry(desktop->screenNumber(q));
+        screenGeom = desktop->screenGeometry(desktop->screenNumber(q));
+    }
+
     QRect dialogGeom = q->geometry();
 
     qreal topHeight(0);
@@ -219,7 +171,8 @@ void DialogPrivate::checkBorders(bool updateMaskIfNeeded)
     qreal bottomHeight(0);
 
     //decide about disabling the border attached to the panel
-    if (applet) {
+    //don't cut borders on tooltips
+    if (applet && !q->testAttribute(Qt::WA_X11NetWmWindowTypeToolTip)) {
         background->getMargins(leftWidth, topHeight, rightWidth, bottomHeight);
 
         switch (applet->location()) {
@@ -271,32 +224,45 @@ void DialogPrivate::checkBorders(bool updateMaskIfNeeded)
     }
 
     //decide if to disable the other borders
-    if (!extender && q->isVisible()) {
-        QRect geom;
-        if (applet) {
-            geom = screenGeom;
-        } else {
-            geom = avail;
-        }
-
-        if (dialogGeom.left() <= geom.left()) {
+    if (q->isVisible() && !q->testAttribute(Qt::WA_X11NetWmWindowTypeToolTip)) {
+        if (dialogGeom.left() <= avail.left()) {
             borders &= ~FrameSvg::LeftBorder;
         }
-        if (dialogGeom.top() <= geom.top()) {
+        if (dialogGeom.top() <= avail.top()) {
             borders &= ~FrameSvg::TopBorder;
         }
         //FIXME: that 2 pixels offset has probably something to do with kwin
-        if (dialogGeom.right() + 2 > geom.right()) {
+        if (dialogGeom.right() + 2 > avail.right()) {
             borders &= ~FrameSvg::RightBorder;
         }
-        if (dialogGeom.bottom() + 2 > geom.bottom()) {
+        if (dialogGeom.bottom() + 2 > avail.bottom()) {
             borders &= ~FrameSvg::BottomBorder;
         }
     }
 
     background->setEnabledBorders(borders);
 
-    if (!extender) {
+    if (extender)  {
+        FrameSvg::EnabledBorders disabledBorders = FrameSvg::NoBorder;
+        if (!(borders & FrameSvg::LeftBorder)) {
+            disabledBorders |= FrameSvg::LeftBorder;
+        }
+        if (!(borders & FrameSvg::RightBorder)) {
+            disabledBorders |= FrameSvg::RightBorder;
+        }
+        extender->d->setDisabledBordersHint(disabledBorders);
+
+        //if there is a scrollbar, reserve a margin to not draw it over the shadow
+        qreal left, top, right, bottom;
+        background->getMargins(left, top, right, bottom);
+        if (extender->d->scrollWidget->viewportGeometry().height() < extender->d->scrollWidget->contentsSize().height()) {
+            if (QApplication::layoutDirection() == Qt::RightToLeft) {
+                leftWidth = left;
+            } else {
+                rightWidth = right;
+            }
+        }
+    } else {
         background->getMargins(leftWidth, topHeight, rightWidth, bottomHeight);
     }
 
@@ -313,7 +279,7 @@ void DialogPrivate::checkBorders(bool updateMaskIfNeeded)
         }
 
         q->update();
-    } 
+    }
 
     resizeChecksWithBorderCheck = false;
 }
@@ -342,21 +308,22 @@ void Dialog::syncToGraphicsWidget()
 
         graphicsWidget->setMaximumSize(maxSize - QSize(left + right, top + bottom).boundedTo(graphicsWidget->effectiveSizeHint(Qt::MaximumSize).toSize()));
 
-        setMinimumSize(-1, -1);
+        setMinimumSize(0, 0);
         setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
 
         QSize newSize(qMin(int(graphicsWidget->size().width()) + left + right, maxSize.width()),
                qMin(int(graphicsWidget->size().height()) + top + bottom, maxSize.height()));
 
-        QSize newMinimumSize(qMin(int(graphicsWidget->minimumSize().width()) + left + right, maxSize.width()),
-                       qMin(int(graphicsWidget->minimumSize().height()) + top + bottom, maxSize.height()));
+        const QSizeF minimum = graphicsWidget->effectiveSizeHint(Qt::MinimumSize);
+        QSize newMinimumSize(qMin(int(minimum.width()) + left + right, maxSize.width()),
+                       qMin(int(minimum.height()) + top + bottom, maxSize.height()));
 
 
         QSize newMaximumSize(qMin(int(graphicsWidget->maximumSize().width()) + left + right, maxSize.width()),
                        qMin(int(graphicsWidget->maximumSize().height()) + top + bottom, maxSize.height()));
 
 
-        Plasma::Applet *applet = d->applet();
+        Plasma::Applet *applet = d->appletPtr.data();
         if (applet) {
             QRect currentGeometry(geometry());
             currentGeometry.setSize(newSize);
@@ -391,14 +358,6 @@ void Dialog::syncToGraphicsWidget()
         //d->view->resize(graphicsWidget->size().toSize());
         d->view->centerOn(graphicsWidget);
 
-        //if the view resized and a border is disabled move the dialog to make sure it will still look attached to panel/screen edge
-        qreal topHeight;
-        qreal leftWidth;
-        qreal rightWidth;
-        qreal bottomHeight;
-
-        d->background->getMargins(leftWidth, topHeight, rightWidth, bottomHeight);
-
         if (size() != prevSize) {
             //the size of the dialog has changed, emit the signal:
             emit dialogResized();
@@ -429,11 +388,13 @@ Dialog::Dialog(QWidget *parent, Qt::WindowFlags f)
     : QWidget(parent, f | Qt::FramelessWindowHint),
       d(new DialogPrivate(this))
 {
+    setMouseTracking(true);
     setAttribute(Qt::WA_TranslucentBackground);
     d->background = new FrameSvg(this);
     d->background->setImagePath("dialogs/background");
     d->background->setEnabledBorders(FrameSvg::AllBorders);
     d->background->resizeFrame(size());
+    connect(d->background, SIGNAL(repaintNeeded()), this, SLOT(themeChanged()));
 
     QPalette pal = palette();
     pal.setColor(backgroundRole(), Qt::transparent);
@@ -448,12 +409,7 @@ Dialog::Dialog(QWidget *parent, Qt::WindowFlags f)
     d->adjustSizeTimer->setSingleShot(true);
     connect(d->adjustSizeTimer, SIGNAL(timeout()), this, SLOT(delayedAdjustSize()));
 
-    connect(d->background, SIGNAL(repaintNeeded()), this, SLOT(update()));
-
-    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeChanged()));
     d->themeChanged();
-
-    setMouseTracking(true);
 }
 
 Dialog::~Dialog()
@@ -535,8 +491,27 @@ void Dialog::mouseMoveEvent(QMouseEvent *event)
                 break;
         }
 
-        if ((newWidth >= minimumSize().width()) && (newHeight >= minimumSize().height())) {
-            setGeometry(QRect(position, QSize(newWidth, newHeight)));
+        QRect newGeom(position, QSize(newWidth, newHeight));
+
+        // now sanity check the resize results again min constraints, if any
+        if (d->leftResizeMin > -1 && newGeom.left() > d->leftResizeMin) {
+            newGeom.setLeft(d->leftResizeMin);
+        }
+
+        if (d->topResizeMin > -1 && newGeom.top() > d->topResizeMin) {
+            newGeom.setTop(d->topResizeMin);
+        }
+
+        if (d->rightResizeMin > -1 && newGeom.right() < d->rightResizeMin) {
+            newGeom.setRight(d->rightResizeMin);
+        }
+
+        if (d->bottomResizeMin > -1 && newGeom.bottom() < d->bottomResizeMin) {
+            newGeom.setBottom(d->bottomResizeMin);
+        }
+
+        if ((newGeom.width() >= minimumSize().width()) && (newGeom.height() >= minimumSize().height())) {
+            setGeometry(newGeom);
         }
     }
 
@@ -563,9 +538,9 @@ void Dialog::mousePressEvent(QMouseEvent *event)
 void Dialog::mouseReleaseEvent(QMouseEvent *event)
 {
     if (d->resizeStartCorner != Dialog::NoCorner) {
+        emit dialogResized();
         d->resizeStartCorner = Dialog::NoCorner;
         unsetCursor();
-        emit dialogResized();
     }
 
     QWidget::mouseReleaseEvent(event);
@@ -580,12 +555,6 @@ void Dialog::keyPressEvent(QKeyEvent *event)
 
 bool Dialog::event(QEvent *event)
 {
-    if (event->type() == QEvent::Paint) {
-        QPainter p(this);
-        p.setCompositionMode(QPainter::CompositionMode_Source);
-        p.fillRect(rect(), Qt::transparent);
-    }
-
     return QWidget::event(event);
 }
 
@@ -656,6 +625,11 @@ void Dialog::setGraphicsWidget(QGraphicsWidget *widget)
     d->graphicsWidgetPtr = widget;
 
     if (widget) {
+        Plasma::Corona *c = qobject_cast<Plasma::Corona *>(widget->scene());
+        if (c) {
+            c->addOffscreenWidget(widget);
+        }
+
         if (!layout()) {
             QVBoxLayout *lay = new QVBoxLayout(this);
             lay->setMargin(0);
@@ -791,9 +765,45 @@ Dialog::ResizeCorners Dialog::resizeCorners() const
     return d->resizeCorners;
 }
 
+bool Dialog::isUserResizing() const
+{
+    return d->resizeStartCorner > NoCorner;
+}
+
+void Dialog::setMinimumResizeLimits(int left, int top, int right, int bottom)
+{
+    d->leftResizeMin = left;
+    d->topResizeMin = top;
+    d->rightResizeMin = right;
+    d->bottomResizeMin = bottom;
+}
+
+void Dialog::getMinimumResizeLimits(int *left, int *top, int *right, int *bottom)
+{
+    if (left) {
+        *left = d->leftResizeMin;
+    }
+
+    if (top) {
+        *top = d->topResizeMin;
+    }
+
+    if (right) {
+        *right = d->rightResizeMin;
+    }
+
+    if (bottom) {
+        *bottom = d->bottomResizeMin;
+    }
+}
+
 void Dialog::animatedHide(Plasma::Direction direction)
 {
-    if (!KWindowSystem::compositingActive()) {
+    if (!isVisible()) {
+        return;
+    }
+
+    if (!Plasma::Theme::defaultTheme()->windowTranslucencyEnabled()) {
         hide();
         return;
     }
@@ -822,7 +832,7 @@ void Dialog::animatedHide(Plasma::Direction direction)
 
 void Dialog::animatedShow(Plasma::Direction direction)
 {
-    if (!KWindowSystem::compositingActive()) {
+    if (!Plasma::Theme::defaultTheme()->windowTranslucencyEnabled()) {
         show();
         return;
     }
@@ -846,7 +856,7 @@ void Dialog::animatedShow(Plasma::Direction direction)
         break;
     }
 
-    if (KWindowSystem::compositingActive()) {
+    if (Plasma::Theme::defaultTheme()->windowTranslucencyEnabled()) {
         Plasma::WindowEffects::slideWindow(this, location);
     }
 
